Index: kern/kern_exit.c
===================================================================
RCS file: /usr/cvs/FreeBSD-CVS/src/sys/kern/kern_exit.c,v
retrieving revision 1.257
diff -u -r1.257 kern_exit.c
--- kern/kern_exit.c	13 Mar 2005 11:47:04 -0000	1.257
+++ kern/kern_exit.c	18 Apr 2005 00:08:30 -0000
@@ -572,6 +572,7 @@
 	return (error);
 }
 
+int fixrace = 1;
 int
 kern_wait(struct thread *td, pid_t pid, int *status, int options,
     struct rusage *rusage)
@@ -739,7 +740,11 @@
 	}
 	PROC_LOCK(q);
 	sx_xunlock(&proctree_lock);
-	error = msleep(q, &q->p_mtx, PWAIT | PCATCH, "wait", 0);
+	if (fixrace == 0 || (q->p_flag & P_STATCHILD) == 0)
+		error = msleep(q, &q->p_mtx, PWAIT | PCATCH, "wait", 0);
+	else
+		error = 0;
+	q->p_flag &= ~P_STATCHILD;
 	PROC_UNLOCK(q);
 	if (error)
 		return (error);	
Index: kern/kern_sig.c
===================================================================
RCS file: /usr/cvs/FreeBSD-CVS/src/sys/kern/kern_sig.c,v
retrieving revision 1.304
diff -u -r1.304 kern_sig.c
--- kern/kern_sig.c	10 Apr 2005 02:31:24 -0000	1.304
+++ kern/kern_sig.c	18 Apr 2005 00:08:31 -0000
@@ -71,6 +71,7 @@
 #include <sys/sysproto.h>
 #include <sys/unistd.h>
 #include <sys/wait.h>
+#include <sys/kdb.h>
 
 #include <machine/cpu.h>
 
@@ -2259,8 +2260,10 @@
 {
 
 	PROC_LOCK_ASSERT(p, MA_OWNED);
+	PROC_LOCK_ASSERT(p->p_pptr, MA_OWNED);
 	p->p_flag |= P_STOPPED_SIG;
 	p->p_flag &= ~P_WAITED;
+	p->p_pptr->p_flag |= P_STATCHILD;
 	wakeup(p->p_pptr);
 }
 
@@ -2281,8 +2284,8 @@
 		n++;
 	if ((p->p_flag & P_STOPPED_SIG) && (n == p->p_numthreads)) {
 		mtx_unlock_spin(&sched_lock);
-		stop(p);
 		PROC_LOCK(p->p_pptr);
+		stop(p);
 		ps = p->p_pptr->p_sigacts;
 		mtx_lock(&ps->ps_mtx);
 		if ((ps->ps_flag & PS_NOCLDSTOP) == 0) {
Index: sys/proc.h
===================================================================
RCS file: /usr/cvs/FreeBSD-CVS/src/sys/sys/proc.h,v
retrieving revision 1.424
diff -u -r1.424 proc.h
--- sys/proc.h	8 Apr 2005 03:37:52 -0000	1.424
+++ sys/proc.h	18 Apr 2005 00:08:44 -0000
@@ -636,6 +636,7 @@
 #define	P_SINGLE_BOUNDARY 0x400000 /* Threads should suspend at user boundary. */
 #define	P_JAILED	0x1000000 /* Process is in jail. */
 #define	P_INEXEC	0x4000000 /* Process is in execve(). */
+#define	P_STATCHILD	0x8000000 /* A child has status to report. */
 
 #define	P_STOPPED	(P_STOPPED_SIG|P_STOPPED_SINGLE|P_STOPPED_TRACE)
 #define	P_SHOULDSTOP(p)	((p)->p_flag & P_STOPPED)
